@@ -19,6 +19,7 @@ import os
 import re
 import sys
 from datetime import datetime, timezone
+from pathlib import Path
 from shutil import copyfileobj
 
 import img2pdf
@@ -37,18 +38,18 @@ from .exceptions import (
     UnsupportedImageFormatError,
 )
 from .exec import ghostscript, tesseract
-from .helpers import re_symlink
+from .helpers import safe_symlink
 from .hocrtransform import HocrTransform
 from .optimize import optimize
 from .pdfa import generate_pdfa_ps
-from .pdfinfo import Colorspace, PdfInfo
+from .pdfinfo import Colorspace, Encoding, PdfInfo
 
 VECTOR_PAGE_DPI = 400
 
 
 def triage_image_file(input_file, output_file, options, log):
+    log.info("Input file is not a PDF, checking if it is an image...")
     try:
-        log.info("Input file is not a PDF, checking if it is an image...")
         im = Image.open(input_file)
     except EnvironmentError as e:
         # Recover the original filename
@@ -85,9 +86,9 @@ def triage_image_file(input_file, output_file, options, log):
 
         if 'iccprofile' not in im.info:
             if im.mode == 'RGB':
-                log.info('Input image has no ICC profile, assuming sRGB')
+                log.info("Input image has no ICC profile, assuming sRGB")
             elif im.mode == 'CMYK':
-                log.info('Input CMYK image has no ICC profile, not usable')
+                log.error("Input CMYK image has no ICC profile, not usable")
                 raise UnsupportedImageFormatError()
 
     try:
@@ -123,7 +124,7 @@ def _pdf_guess_version(input_file, search_window=1024):
     return ''
 
 
-def triage(input_file, output_file, options, log):
+def triage(original_filename, input_file, output_file, options, log):
     try:
         if _pdf_guess_version(input_file):
             if options.image_dpi:
@@ -132,11 +133,12 @@ def triage(input_file, output_file, options, log):
                     "input file is a PDF, not an image."
                 )
             # Origin file is a pdf create a symlink with pdf extension
-            re_symlink(input_file, output_file)
+            safe_symlink(input_file, output_file)
             return output_file
     except EnvironmentError as e:
-        log.error(e)
-        raise InputFileError() from e
+        log.debug(f"Temporary file was at: {input_file}")
+        msg = str(e).replace(input_file, original_filename)
+        raise InputFileError(msg) from e
 
     triage_image_file(input_file, output_file, options, log)
     return output_file
@@ -181,7 +183,7 @@ def validate_pdfinfo_options(context):
             )
             raise InputFileError()
         else:
-            log.warn(
+            log.warning(
                 "This PDF has a fillable form. "
                 "Chances are it is a pure digital "
                 "document that does not need OCR."
@@ -557,7 +559,7 @@ def ocr_tesseract_hocr(input_file, page_context):
 
 def should_visible_page_image_use_jpg(pageinfo):
     # If all images were JPEGs originally, produce a JPEG as output
-    return pageinfo.images and all(im.enc == 'jpeg' for im in pageinfo.images)
+    return pageinfo.images and all(im.enc == Encoding.jpeg for im in pageinfo.images)
 
 
 def create_visible_page_jpg(image, page_context):
@@ -701,7 +703,7 @@ def convert_to_pdfa(input_pdf, input_ps_stub, context):
         if modified:
             pdf_file.save(fix_docinfo_file)
         else:
-            os.symlink(input_pdf, fix_docinfo_file)
+            safe_symlink(input_pdf, fix_docinfo_file)
 
     ghostscript.generate_pdfa(
         pdf_version=input_pdfinfo.min_version,
@@ -725,47 +727,53 @@ def should_linearize(working_file, context):
 def metadata_fixup(working_file, context):
     output_file = context.get_path('metafix.pdf')
     options = context.options
-    original = pikepdf.open(context.origin)
-    docinfo = get_docinfo(original, options)
-    pdf = pikepdf.open(working_file)
-    with pdf.open_metadata() as meta:
-        meta.load_from_docinfo(docinfo, delete_missing=False)
-        # If xmp:CreateDate is missing, set it to the modify date to
-        # match Ghostscript, for consistency
-        if 'xmp:CreateDate' not in meta:
-            meta['xmp:CreateDate'] = meta.get('xmp:ModifyDate', '')
 
-        meta_original = original.open_metadata()
-        not_copied = set(meta_original.keys()) - set(meta.keys())
-        if not_copied:
-            if options.output_type.startswith('pdfa'):
-                context.log.warning(
-                    "Some input metadata could not be copied because it is not "
-                    "permitted in PDF/A. You may wish to examine the output "
-                    "PDF's XMP metadata."
-                )
-                context.log.debug(
-                    "The following metadata fields were not copied: %r", not_copied
-                )
-            else:
-                context.log.error(
-                    "Some input metadata could not be copied."
-                    "You may wish to examine the output PDF's XMP metadata."
-                )
-                context.log.info(
-                    "The following metadata fields were not copied: %r", not_copied
-                )
-    pdf.save(
-        output_file,
-        compress_streams=True,
-        preserve_pdfa=True,
-        object_stream_mode=pikepdf.ObjectStreamMode.generate,
-        linearize=(  # Don't linearize if optimize() will be linearizing too
-            should_linearize(working_file, context) if options.optimize == 0 else False
-        ),
-    )
-    original.close()
-    pdf.close()
+    def report_on_metadata(missing):
+        if not missing:
+            return
+        if options.output_type.startswith('pdfa'):
+            context.log.warning(
+                "Some input metadata could not be copied because it is not "
+                "permitted in PDF/A. You may wish to examine the output "
+                "PDF's XMP metadata."
+            )
+            context.log.debug(
+                "The following metadata fields were not copied: %r", missing
+            )
+        else:
+            context.log.error(
+                "Some input metadata could not be copied."
+                "You may wish to examine the output PDF's XMP metadata."
+            )
+            context.log.info(
+                "The following metadata fields were not copied: %r", missing
+            )
+
+    with pikepdf.open(context.origin) as original, pikepdf.open(working_file) as pdf:
+        docinfo = get_docinfo(original, options)
+        with pdf.open_metadata() as meta:
+            meta.load_from_docinfo(docinfo, delete_missing=False, raise_failure=False)
+            # If xmp:CreateDate is missing, set it to the modify date to
+            # match Ghostscript, for consistency
+            if 'xmp:CreateDate' not in meta:
+                meta['xmp:CreateDate'] = meta.get('xmp:ModifyDate', '')
+
+            meta_original = original.open_metadata()
+            missing = set(meta_original.keys()) - set(meta.keys())
+            report_on_metadata(missing)
+
+        pdf.save(
+            output_file,
+            compress_streams=True,
+            preserve_pdfa=True,
+            object_stream_mode=pikepdf.ObjectStreamMode.generate,
+            linearize=(  # Don't linearize if optimize() will be linearizing too
+                should_linearize(working_file, context)
+                if options.optimize == 0
+                else False
+            ),
+        )
+
     return output_file
 
 
